"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

exports.__esModule = true;
exports.useLazyHydration = useLazyHydration;

var _react = _interopRequireWildcard(require("react"));

function useLazyHydration(component, props) {
  var ssrOnly = props.ssrOnly,
      whenIdle = props.whenIdle,
      whenVisible = props.whenVisible;

  if (!ssrOnly && !whenIdle && !whenVisible) {
    console.warn("LazyHydrate: Set atleast one of the props to 'true'");
  }

  var _useState = (0, _react.useState)(typeof window === "undefined"),
      hydrated = _useState[0],
      setHydrated = _useState[1];

  var childRef = (0, _react.useRef)(null);
  var cleanupFns = (0, _react.useRef)([]);

  function cleanup() {
    while (cleanupFns.current.length > 0) {
      cleanupFns.current.pop()();
    }
  }

  function hydrate() {
    setHydrated(true);
    cleanup();
  }

  var io = (0, _react.useRef)(null);
  (0, _react.useLayoutEffect)(function () {
    if (childRef.current.childElementCount === 0) {
      // No SSR rendered content.
      hydrate();
      return;
    }

    if (ssrOnly) return;

    if (whenIdle) {
      // @ts-ignore
      if (window.requestIdleCallback) {
        // @ts-ignore
        var id = window.requestIdleCallback(function () {
          return requestAnimationFrame(function () {
            return hydrate();
          });
        }, {
          timeout: 500
        }); // @ts-ignore

        cleanupFns.current.push(function () {
          return cancelIdleCallback(id);
        });
      } else {
        hydrate();
        return;
      }
    }

    if (whenVisible) {
      if (typeof IntersectionObserver !== "undefined") {
        io.current = new IntersectionObserver(function (entries) {
          entries.forEach(function (entry) {
            if (entry.target.parentElement === childRef.current && (entry.isIntersecting || entry.intersectionRatio > 0)) {
              hydrate();
            }
          });
        });
        io.current.observe(childRef.current.children[0]);
        cleanupFns.current.push(function () {
          return io.current.unobserve(childRef.current.children[0]);
        });
      } else {
        hydrate();
      }
    }

    return cleanup;
  }, []);
  var lazyComponent = hydrated ? _react.default.createElement("div", {
    ref: childRef,
    style: {
      display: "contents"
    }
  }, component) : _react.default.createElement("div", {
    ref: childRef,
    style: {
      display: "contents"
    },
    dangerouslySetInnerHTML: {
      __html: ""
    }
  });
  return [lazyComponent, hydrated];
}